<?php
/**
 * Plugin Name: Click Tracker S
 * Description: 指定したクラスを持つリンクのクリックを計測するプラグイン。
 * Version: 1.0
 * Author: Sifr
 * Author URI: https://www.sifrinsight.com/
 * Text Domain:click tracker s
 * Tested up to:6.7
 * License: GPL2 or later
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 
*/

if (!defined('ABSPATH')) {
    exit;
}

class ClickTrackerPlugin {
    public function __construct() {
        // プラグインの有効化時にテーブルを作成
        register_activation_hook(__FILE__, array($this, 'install'));

        // アクションフックとフィルターを追加
        add_action('admin_menu', array($this, 'add_admin_menu'));
        add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_scripts'));
        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));

        // フロントエンドスクリプトに defer 属性を追加
        add_filter('script_loader_tag', array($this, 'add_defer_attribute'), 10, 3);

        // AJAX ハンドラー
        add_action('wp_ajax_click_tracker_track_click', array($this, 'track_click'));
        add_action('wp_ajax_nopriv_click_tracker_track_click', array($this, 'track_click'));
        add_action('wp_ajax_click_tracker_get_graph_data', array($this, 'get_graph_data'));
        add_action('wp_ajax_click_tracker_get_aggregation_results', array($this, 'get_aggregation_results'));
    }

    public function install() {
        global $wpdb;
        $table_name = $wpdb->prefix . 'click_tracker';
        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE $table_name (
            id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
            class_name VARCHAR(255) NOT NULL,
            tag VARCHAR(255) NOT NULL,
            group_name VARCHAR(255) DEFAULT NULL,
            url TEXT NOT NULL,
            ip_address VARCHAR(100) NOT NULL,
            user_agent TEXT NOT NULL,
            clicked_at DATETIME NOT NULL,
            PRIMARY KEY (id),
            INDEX (tag),
            INDEX (group_name),
            INDEX (clicked_at)
        ) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    public function add_admin_menu() {
        add_menu_page(
            'Click Tracker S',
            'Click Tracker S',
            'manage_options',
            'click-tracker',
            array($this, 'display_aggregation_results_page'),
            'dashicons-chart-bar',
            6
        );

        add_submenu_page(
            'click-tracker',
            'Click Tracker 設定',
            '設定',
            'manage_options',
            'click-tracker-settings',
            array($this, 'display_settings_page')
        );
    }

    public function admin_enqueue_scripts($hook) {
        if (strpos($hook, 'click-tracker') !== false) {
            wp_enqueue_style('click-tracker-admin', plugin_dir_url(__FILE__) . 'admin.css');
            wp_enqueue_script('click-tracker-admin', plugin_dir_url(__FILE__) . 'admin.js', array('jquery'), '1.0', true);
            wp_enqueue_script('chartjs', 'https://cdn.jsdelivr.net/npm/chart.js', array(), '3.7.1', true);
        }
    }

    public function enqueue_scripts() {
        wp_enqueue_script('click-tracker-frontend', plugin_dir_url(__FILE__) . 'frontend.js', array(), '1.0', true);

        $settings = get_option('click_tracker_settings', array());
        $track_admin = isset($settings['track_admin']) ? $settings['track_admin'] : false;

        if (!current_user_can('manage_options') || $track_admin) {
            $tracking_classes = isset($settings['tracking_classes']) ? $settings['tracking_classes'] : array();
            $exclude_ips = isset($settings['exclude_ips']) ? $settings['exclude_ips'] : array();

            wp_localize_script('click-tracker-frontend', 'clickTrackerData', array(
                'ajax_url' => admin_url('admin-ajax.php'),
                'tracking_classes' => $tracking_classes,
                'exclude_ips' => $exclude_ips,
                'nonce' => wp_create_nonce('click_tracker_nonce'),
            ));
        }
    }

    public function add_defer_attribute($tag, $handle, $src) {
        if ('click-tracker-frontend' !== $handle) {
            return $tag;
        }
        // defer 属性を追加
        return '<script src="' . esc_url($src) . '" defer></script>';
    }

    public function track_click() {
        check_ajax_referer('click_tracker_nonce', 'security');

        $class_name = sanitize_text_field($_POST['class_name']);
        $tag = sanitize_text_field($_POST['tag']);
        $group_name = sanitize_text_field($_POST['group_name']);
        $url = esc_url_raw($_POST['url']);

        $settings = get_option('click_tracker_settings', array());
        $exclude_ips = isset($settings['exclude_ips']) ? $settings['exclude_ips'] : array();
        $user_ip = $_SERVER['REMOTE_ADDR'];

        if (in_array($user_ip, $exclude_ips)) {
            wp_send_json_error('IP address excluded');
        }

        if (!isset($settings['track_admin']) || !$settings['track_admin']) {
            if (current_user_can('manage_options')) {
                wp_send_json_error('Admin not tracked');
            }
        }

        global $wpdb;
        $table_name = $wpdb->prefix . 'click_tracker';

        $data = array(
            'class_name' => $class_name,
            'tag' => $tag,
            'group_name' => $group_name,
            'url' => $url,
            'ip_address' => $user_ip,
            'user_agent' => $_SERVER['HTTP_USER_AGENT'],
            'clicked_at' => current_time('mysql'),
        );

        $wpdb->insert($table_name, $data);

        wp_send_json_success('Click tracked');
    }

    public function get_graph_data() {
        check_ajax_referer('click_tracker_nonce', 'security');

        $tag = sanitize_text_field($_POST['tag']);
        $period = sanitize_text_field($_POST['period']);

        global $wpdb;
        $table_name = $wpdb->prefix . 'click_tracker';

        $current_time = current_time('timestamp');
        $labels = array();
        $data_points = array();

        switch ($period) {
            case 'today':
            case 'yesterday':
                $hours = array();
                for ($i = 0; $i < 24; $i++) {
                    $hours[] = str_pad($i, 2, '0', STR_PAD_LEFT) . ':00';
                }
                $labels = $hours;

                if ($period == 'today') {
                    $start_time = date('Y-m-d 00:00:00', $current_time);
                    $end_time = date('Y-m-d 23:59:59', $current_time);
                } else {
                    $start_time = date('Y-m-d 00:00:00', strtotime('-1 day', $current_time));
                    $end_time = date('Y-m-d 23:59:59', strtotime('-1 day', $current_time));
                }

                $results = $wpdb->get_results($wpdb->prepare("
                    SELECT HOUR(clicked_at) as hour, COUNT(*) as count
                    FROM $table_name
                    WHERE tag = %s AND clicked_at BETWEEN %s AND %s
                    GROUP BY HOUR(clicked_at)
                ", $tag, $start_time, $end_time));

                $data_points = array_fill(0, 24, 0);
                foreach ($results as $row) {
                    $data_points[$row->hour] = (int)$row->count;
                }
                break;

            case '7_days':
            case '30_days':
                $days = ($period == '7_days') ? 7 : 30;
                for ($i = $days - 1; $i >= 0; $i--) {
                    $labels[] = date('m/d', strtotime("-$i day", $current_time));
                }

                $start_time = date('Y-m-d 00:00:00', strtotime("-$days days +1 day", $current_time));
                $results = $wpdb->get_results($wpdb->prepare("
                    SELECT DATE(clicked_at) as date, COUNT(*) as count
                    FROM $table_name
                    WHERE tag = %s AND clicked_at >= %s
                    GROUP BY DATE(clicked_at)
                ", $tag, $start_time));

                $data_points = array_fill(0, $days, 0);
                foreach ($results as $row) {
                    $index = array_search(date('m/d', strtotime($row->date)), $labels);
                    if ($index !== false) {
                        $data_points[$index] = (int)$row->count;
                    }
                }
                break;

            case '12_months':
                for ($i = 11; $i >= 0; $i--) {
                    $labels[] = date('Y-m', strtotime("-$i month", $current_time));
                }

                $start_time = date('Y-m-01 00:00:00', strtotime('-11 months', $current_time));
                $results = $wpdb->get_results($wpdb->prepare("
                    SELECT DATE_FORMAT(clicked_at, '%%Y-%%m') as month, COUNT(*) as count
                    FROM $table_name
                    WHERE tag = %s AND clicked_at >= %s
                    GROUP BY DATE_FORMAT(clicked_at, '%%Y-%%m')
                ", $tag, $start_time));

                $data_points = array_fill(0, 12, 0);
                foreach ($results as $row) {
                    $index = array_search($row->month, $labels);
                    if ($index !== false) {
                        $data_points[$index] = (int)$row->count;
                    }
                }
                break;
        }

        $response = array(
            'labels' => $labels,
            'datasets' => array(
                array(
                    'label' => 'クリック数',
                    'data' => $data_points,
                    'backgroundColor' => 'rgba(75, 192, 192, 0.2)',
                    'borderColor' => 'rgba(75, 192, 192, 1)',
                    'borderWidth' => 1
                )
            )
        );

        wp_send_json_success($response);
    }

    public function get_aggregation_results() {
        check_ajax_referer('click_tracker_nonce', 'security');

        $aggregation_target = sanitize_text_field($_POST['aggregation_target']);
        $aggregation_period = sanitize_text_field($_POST['aggregation_period']);
        $selected_tab = sanitize_text_field($_POST['selected_tab']);

        global $wpdb;
        $table_name = $wpdb->prefix . 'click_tracker';

        $settings = get_option('click_tracker_settings', array());
        $group_presets = isset($settings['group_presets']) ? $settings['group_presets'] : array();

        $time_condition = '';
        $current_time = current_time('timestamp');
        if ($aggregation_period == 'today') {
            $start_time = date('Y-m-d 00:00:00', $current_time);
            $time_condition = $wpdb->prepare("AND clicked_at >= %s", $start_time);
        } elseif ($aggregation_period == 'yesterday') {
            $start_time = date('Y-m-d 00:00:00', strtotime('-1 day', $current_time));
            $end_time = date('Y-m-d 23:59:59', strtotime('-1 day', $current_time));
            $time_condition = $wpdb->prepare("AND clicked_at BETWEEN %s AND %s", $start_time, $end_time);
        } elseif ($aggregation_period == '7_days') {
            $start_time = date('Y-m-d 00:00:00', strtotime('-6 days', $current_time));
            $time_condition = $wpdb->prepare("AND clicked_at >= %s", $start_time);
        } elseif ($aggregation_period == '30_days') {
            $start_time = date('Y-m-d 00:00:00', strtotime('-29 days', $current_time));
            $time_condition = $wpdb->prepare("AND clicked_at >= %s", $start_time);
        } elseif ($aggregation_period == '12_months') {
            $start_time = date('Y-m-01 00:00:00', strtotime('-11 months', $current_time));
            $time_condition = $wpdb->prepare("AND clicked_at >= %s", $start_time);
        } else {
            $start_time = date('Y-m-d 00:00:00', $current_time);
            $time_condition = $wpdb->prepare("AND clicked_at >= %s", $start_time);
        }

        $results = array();
        if ($aggregation_target == 'tags') {
            $query = "SELECT tag, COUNT(*) as count FROM $table_name WHERE 1=1 $time_condition GROUP BY tag ORDER BY count DESC";
        } else {
            $query = "SELECT tag, url, COUNT(*) as count FROM $table_name WHERE 1=1 $time_condition GROUP BY tag, url ORDER BY count DESC";
        }
        $rows = $wpdb->get_results($query);

        $data_by_group = array();
        foreach ($rows as $row) {
            $tag = $row->tag;
            $count = $row->count;
            $url = isset($row->url) ? $row->url : '';

            $group_name = '';
            foreach ($settings['tracking_classes'] as $tracking_class) {
                if ($tracking_class['tag'] == $tag) {
                    $group_name = $tracking_class['group_name'];
                    break;
                }
            }

            if (!isset($data_by_group[$group_name])) {
                $data_by_group[$group_name] = array();
            }
            $data_entry = array('tag' => $tag, 'count' => $count);
            if ($aggregation_target == 'tags_urls') {
                $data_entry['url'] = $url;
            }
            $data_by_group[$group_name][] = $data_entry;
        }

        $all_data = array();
        foreach ($data_by_group as $group_name => $group_data) {
            foreach ($group_data as $data_entry) {
                $key = $aggregation_target == 'tags' ? $data_entry['tag'] : $data_entry['tag'] . $data_entry['url'];
                if (!isset($all_data[$key])) {
                    $all_data[$key] = $data_entry;
                } else {
                    $all_data[$key]['count'] += $data_entry['count'];
                }
            }
        }

        if ($selected_tab == 'all') {
            $data_to_display = array_values($all_data);
        } else {
            $data_to_display = isset($data_by_group[$selected_tab]) ? $data_by_group[$selected_tab] : array();
        }

        wp_send_json_success(array(
            'data_to_display' => $data_to_display,
            'aggregation_target' => $aggregation_target,
            'group_presets' => $group_presets,
        ));
    }

    public function display_aggregation_results_page() {
        if (!current_user_can('manage_options')) {
            return;
        }

        include plugin_dir_path(__FILE__) . 'aggregation-results-page.php';
    }

    public function display_settings_page() {
        if (!current_user_can('manage_options')) {
            return;
        }

        include plugin_dir_path(__FILE__) . 'settings-page.php';
    }
}

new ClickTrackerPlugin();
